@@ -1,9 +1,9 @@
 package org.jboss.as.test.integration.web.handlestypes;
 
-import javax.servlet.ServletContainerInitializer;
-import javax.servlet.ServletContext;
-import javax.servlet.ServletException;
-import javax.servlet.annotation.HandlesTypes;
+import jakarta.servlet.ServletContainerInitializer;
+import jakarta.servlet.ServletContext;
+import jakarta.servlet.ServletException;
+import jakarta.servlet.annotation.HandlesTypes;
 import java.util.Set;
 import java.util.concurrent.CopyOnWriteArraySet;
 
